/*************************************************************************
 * LICENSE AGREEMENT FOR SAMPLE CODE OR ADDITIONAL FUNCTIONALITY
 * SAS INSTITUTE INC. IS PROVIDING YOU WITH THE COMPUTER SOFTWARE CODE INCLUDED WITH
 * THIS AGREEMENT ("CODE") ON AN "AS IS" BASIS, AND AUTHORIZES YOU TO USE THE CODE SUBJECT
 * TO THE TERMS HEREOF. BY USING THE CODE, YOU AGREE TO THESE TERMS. YOUR USE OF THE CODE
 * IS AT YOUR OWN RISK. SAS INSTITUTE INC. MAKES NO REPRESENTATION OR WARRANTY, EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, NONINFRINGEMENT AND TITLE, WITH RESPECT TO THE CODE.
 *
 * The Code is intended to be used solely as an example for use with part of a product ("Software")
 * you currently have licensed from SAS Institute Inc. or one of its subsidiaries or authorized
 * agents ("SAS"). The Code is designed to add functionality to the Software, but has not necessarily
 * been tested. Accordingly, SAS makes no representation or warranty that the Code will operate
 * error-free. SAS is under no obligation to maintain or support the Code.
 *
 * Neither SAS nor its licensors shall be liable to you or any third party for any general, special,
 * direct, indirect, consequential, incidental or other damages whatsoever arising out of or related
 * to your use or inability to use the Code, even if SAS has been advised of the possibility of such
 * damages.
 *
 * Except as otherwise provided above, the Code is governed by the same agreement that governs the
 * Software. If you do not have an existing agreement with SAS governing the Software, you may not
 * use the Code.
 *
 * SAS and all other SAS Institute Inc. product or service names are registered trademarks or
 * trademarks of SAS Institute Inc. in the USA and other countries. (r) indicates USA registration.
 * Other brand and product names are registered trademarks or trademarks of their respective companies.
 *
 * Copyright 2007, SAS Institute Inc., Cary, NC
 *************************************************************************/
package com.sas.drugdev.remote;

import java.io.InputStream;
import java.util.Iterator;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;

import com.sas.drugdev.remote.repository.AclRecurseBehavior;
import com.sas.drugdev.remote.repository.ContainerBean;
import com.sas.drugdev.remote.repository.ContainerDepth;
import com.sas.drugdev.remote.repository.RemoteContainer;
import com.sas.drugdev.remote.repository.RepositoryService;
import com.sas.drugdev.remote.repository.security.AccessControlEntry;
import com.sas.drugdev.remote.repository.security.AccessControlList;
import com.sas.drugdev.remote.repository.security.Permission;
import com.sas.drugdev.remote.session.Session;
import com.sas.drugdev.remote.type.TypeBean;

class CreateTree {

	private RepositoryService rs;
	

	CreateTree(Session session) {
		this.rs = session.getServiceManager().getRepositoryService();
	}


	private void createContainer(RepositoryService rs, String basepath, Element containerElement) 
	throws Exception {
		if (!basepath.endsWith("/")) basepath = basepath + "/";
		String path = basepath + containerElement.getAttributeValue(Hierarchy.NAME);
		String type = containerElement.getAttributeValue(Hierarchy.TYPE);
		ContainerBean createBean = new ContainerBean(path, new TypeBean(type));
		String owner = containerElement.getAttributeValue(Hierarchy.OWNER);
		for (Iterator iterator = containerElement.getChildren(Hierarchy.PROPERTY).iterator(); iterator.hasNext();) {
			Element propElement = (Element) iterator.next();
			createBean.setProperty(propElement.getAttributeValue(Hierarchy.PROPERTY_NAME), propElement.getAttributeValue(Hierarchy.PROPERTY_VALUE));
		}
		RemoteContainer rc = rs.createContainer(createBean);
		rc = rs.getContainer(new ContainerBean(path));
		String defVersoiningStr = containerElement.getAttributeValue(Hierarchy.DEFAULT_VERSIONING);
		boolean defaultVersioning = ((null != defVersoiningStr)&&Boolean.valueOf(defVersoiningStr).booleanValue());
		if (rc.isDefaultVersioningEnabled()!=defaultVersioning) {
			rc = rs.setDefaultContainerVersioning(rc, ContainerDepth.DEPTH_ZERO, defaultVersioning);
		}
		Element aclElement = containerElement.getChild(Hierarchy.ACL);
		if (aclElement != null) {
			updateAcl(rs, rc, aclElement);
		}		
		if (owner != null) {
			rs.setOwnerOnContainer(new ContainerBean(path), ContainerDepth.DEPTH_ZERO, owner);			
		}
		for (Iterator iter = containerElement.getChildren(Hierarchy.CONTAINER).iterator(); iter.hasNext();) {
			Element subContainerElement = (Element) iter.next();
			createContainer(rs, path, subContainerElement);
		}
	}
	

	private void updateAcl(RepositoryService rs, RemoteContainer rc, Element aclElement) 
	throws Exception {
		AccessControlList acl = rs.getAccessControlList(rc);

		// Remove all existing aces.
		for (Iterator iterator = acl.getEntries().iterator(); iterator
				.hasNext();) {
			AccessControlEntry ace = (AccessControlEntry) iterator.next();
			if (ace.isGroupEntry()) {
				if (isAllUserEntry(ace)) {
					removeAllPermissions(ace);
				} else {
					acl.removeGroupEntry(ace.getName());
				}
			} else {
				if (isOwnerEntry(ace)) {
					removeAllPermissions(ace);
				} else {
					acl.removeUserEntry(ace.getName());
				}
			}
		}
		
		// Add aces from xml doc
		for (Iterator iterator = aclElement.getChildren(Hierarchy.ACE).iterator(); iterator
				.hasNext();) {
			Element aceElement = (Element) iterator.next();
			String aceType = aceElement.getAttributeValue(Hierarchy.ACE_TYPE);
			String principal = aceElement.getAttributeValue(Hierarchy.PRINCIPAL);
			AccessControlEntry ace = null;
			boolean needsAdding = false;
			if (Hierarchy.ACE_GROUP.equals(aceType)) {
				if (Hierarchy.ALL_USERS_GROUP.equals(principal)) {
					ace = acl.getGroupEntry(AccessControlEntry.GROUP_ALL_USERS);
				} else {
					needsAdding = true;
					ace = acl.createGroupEntry(principal);
				}
			} else if (Hierarchy.ACE_USER.equals(aceType)) {
				if (Hierarchy.OWNER_USER.equals(principal)) {
					ace = acl.getUserEntry(AccessControlEntry.USER_OWNER);
				} else {
					needsAdding = true;
					ace = acl.createUserEntry(principal);
				}
			} else {
				throw new Exception("Bad ace type: " + aceType);
			}
			
			for (Iterator aceIter = aceElement.getChildren(Hierarchy.PERMISSION).iterator(); aceIter
					.hasNext();) {
				Element permElement = (Element) aceIter.next();
				String permStr = permElement.getAttributeValue(Hierarchy.PERM_NAME);
				if (Hierarchy.READ_PERM.equals(permStr)) {
					ace.addPermission(Permission.READ_PERMISSION);
				} else if (Hierarchy.WRITE_PERM.equals(permStr)) {
					ace.addPermission(Permission.WRITE_PERMISSION);
				} else if (Hierarchy.DELETE_PERM.equals(permStr)) {
					ace.addPermission(Permission.DELETE_PERMISSION);
				} else if (Hierarchy.MANAGE_PERM.equals(permStr)) {
					ace.addPermission(Permission.MANAGE_PERMISSION);
				} else if (Hierarchy.INHERIT_READ_PERM.equals(permStr)) {
					ace.addPermission(Permission.INHERIT_READ_PERMISSION);
				} else if (Hierarchy.INHERIT_WRITE_PERM.equals(permStr)) {
					ace.addPermission(Permission.INHERIT_WRITE_PERMISSION);
				} else if (Hierarchy.INHERIT_DELETE_PERM.equals(permStr)) {
					ace.addPermission(Permission.INHERIT_DELETE_PERMISSION);
				} else if (Hierarchy.INHERIT_MANAGE_PERM.equals(permStr)) {
					ace.addPermission(Permission.INHERIT_MANAGE_PERMISSION);
				}
			}
			if (needsAdding) acl.addEntry(ace);
		}
		rs.updateContainerAccessControlList(acl, AclRecurseBehavior.RECURSE_NONE);
	}

	private boolean isOwnerEntry(AccessControlEntry ace) {
		return ace.getName().equals(AccessControlEntry.USER_OWNER);
	}

	private boolean isAllUserEntry(AccessControlEntry ace) {
		return ace.getName().equals(AccessControlEntry.GROUP_ALL_USERS);
	}

	private void removeAllPermissions(AccessControlEntry ace) {
		ace.removePermission(Permission.DELETE_PERMISSION);
		ace.removePermission(Permission.WRITE_PERMISSION);
		ace.removePermission(Permission.READ_PERMISSION);
		ace.removePermission(Permission.MANAGE_PERMISSION);
		ace.removePermission(Permission.INHERIT_DELETE_PERMISSION);
		ace.removePermission(Permission.INHERIT_WRITE_PERMISSION);
		ace.removePermission(Permission.INHERIT_READ_PERMISSION);
		ace.removePermission(Permission.INHERIT_MANAGE_PERMISSION);
	}
	

	void create(String sddpath, InputStream xml) throws Exception {
		Document doc = new SAXBuilder().build(xml);
		Element root = doc.getRootElement();
		create(sddpath, root);
	}
	
	void create(String sddpath, Element root) throws Exception {
		RemoteContainer rc = rs.getContainer(new ContainerBean(sddpath));
		if (rc == null) {
			throw new Exception("SDD path doesn't exist: " + sddpath);
		}
		if (!root.getName().equals(Hierarchy.HIERARCHY)) throw new Exception("Bad xml");
		for (Iterator iter = root.getChildren(Hierarchy.CONTAINER).iterator(); iter.hasNext();) {
			Element containerElement = (Element) iter.next();
			createContainer(rs, sddpath, containerElement);	
		}
	}
}
