/*************************************************************************
 * LICENSE AGREEMENT FOR SAMPLE CODE OR ADDITIONAL FUNCTIONALITY
 * SAS INSTITUTE INC. IS PROVIDING YOU WITH THE COMPUTER SOFTWARE CODE INCLUDED WITH
 * THIS AGREEMENT ("CODE") ON AN "AS IS" BASIS, AND AUTHORIZES YOU TO USE THE CODE SUBJECT
 * TO THE TERMS HEREOF. BY USING THE CODE, YOU AGREE TO THESE TERMS. YOUR USE OF THE CODE
 * IS AT YOUR OWN RISK. SAS INSTITUTE INC. MAKES NO REPRESENTATION OR WARRANTY, EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, NONINFRINGEMENT AND TITLE, WITH RESPECT TO THE CODE.
 *
 * The Code is intended to be used solely as an example for use with part of a product ("Software")
 * you currently have licensed from SAS Institute Inc. or one of its subsidiaries or authorized
 * agents ("SAS"). The Code is designed to add functionality to the Software, but has not necessarily
 * been tested. Accordingly, SAS makes no representation or warranty that the Code will operate
 * error-free. SAS is under no obligation to maintain or support the Code.
 *
 * Neither SAS nor its licensors shall be liable to you or any third party for any general, special,
 * direct, indirect, consequential, incidental or other damages whatsoever arising out of or related
 * to your use or inability to use the Code, even if SAS has been advised of the possibility of such
 * damages.
 *
 * Except as otherwise provided above, the Code is governed by the same agreement that governs the
 * Software. If you do not have an existing agreement with SAS governing the Software, you may not
 * use the Code.
 *
 * SAS and all other SAS Institute Inc. product or service names are registered trademarks or
 * trademarks of SAS Institute Inc. in the USA and other countries. (r) indicates USA registration.
 * Other brand and product names are registered trademarks or trademarks of their respective companies.
 *
 * Copyright 2007, SAS Institute Inc., Cary, NC
 *************************************************************************/

package com.sas.drugdev.remote;

import java.io.OutputStream;
import java.net.URLDecoder;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import com.sas.drugdev.remote.repository.ContainerBean;
import com.sas.drugdev.remote.repository.RemoteContainer;
import com.sas.drugdev.remote.repository.RemoteNode;
import com.sas.drugdev.remote.repository.RepositoryService;
import com.sas.drugdev.remote.repository.security.AccessControlEntry;
import com.sas.drugdev.remote.repository.security.AccessControlList;
import com.sas.drugdev.remote.repository.security.Permission;
import com.sas.drugdev.remote.session.Session;

class ExtractTree {
    private static Log log = LogFactory.getLog(ExtractTree.class.getName());
	private RepositoryService rs;
	
	ExtractTree(Session session) {
		this.rs = session.getServiceManager().getRepositoryService();
	}
	
	void extract(String sddpath, OutputStream out) throws Exception {
		Element root = extract(sddpath);
		new XMLOutputter(Format.getPrettyFormat()).output(root, out);
	}

	Element extract(String sddpath) throws Exception {
		Element ret = new Element(Hierarchy.HIERARCHY);
		RemoteContainer rc = rs.getContainer(new ContainerBean(sddpath));
		if (rc == null) {
			throw new Exception("Path doesn't exist: " + sddpath);
		}
		String basepath = sddpath;
		int index = basepath.lastIndexOf('/');
		if (index == -1) {
			throw new Exception("Bad path: " + sddpath);			
		}
		basepath = sddpath.substring(0, index-1);
		ret.addContent(getContainerElement(rc));
		return ret;
	}
	
	private Element getContainerElement(RemoteContainer rc) throws Exception {
		log.debug("Creating element for: " + rc.getPath());
		Element ret = new Element(Hierarchy.CONTAINER);
		ret.setAttribute(Hierarchy.NAME, rc.getName());
		ret.setAttribute(Hierarchy.OWNER, rc.getOwner());
		ret.setAttribute(Hierarchy.TYPE, rc.getType());
		Map props = rc.getProperties();
		for (Iterator iter = props.keySet().iterator(); iter.hasNext();) {
			String key = (String) iter.next();
			Element propElement = new Element(Hierarchy.PROPERTY);
			propElement.setAttribute(Hierarchy.PROPERTY_NAME, key);
			propElement.setAttribute(Hierarchy.PROPERTY_VALUE, (String) props.get(key));
			ret.addContent(propElement);
		}
		if (rc.isDefaultVersioningEnabled()) {
			ret.setAttribute(Hierarchy.DEFAULT_VERSIONING, "true");
		}
		Element aclElement = new Element(Hierarchy.ACL);
		AccessControlList acl = rs.getAccessControlList(rc);
		for (Iterator iter = acl.getEntries().iterator(); iter.hasNext();) {
			AccessControlEntry ace = (AccessControlEntry) iter.next();
			Element aceElement = new Element(Hierarchy.ACE);
			String type = ace.isGroupEntry() ? Hierarchy.ACE_GROUP : Hierarchy.ACE_USER;
			aceElement.setAttribute(Hierarchy.ACE_TYPE, type);
			String principal = ace.getName();
			
			// HACK TO FIX API BUG
			//principal = principal.replaceAll("%20", " ");
			principal = URLDecoder.decode(principal, "UTF-8");

			if (AccessControlEntry.GROUP_ALL_USERS.equals(principal)) {
				principal = Hierarchy.ALL_USERS_GROUP;
			} else if (AccessControlEntry.USER_OWNER.equals(principal)) {
				principal = Hierarchy.OWNER_USER;
			}
			aceElement.setAttribute(Hierarchy.PRINCIPAL, principal);
			if (ace.hasPermission(Permission.READ_PERMISSION)) {
				Element permElement = new Element(Hierarchy.PERMISSION);
				permElement.setAttribute(Hierarchy.PERM_NAME, Hierarchy.READ_PERM);
				aceElement.addContent(permElement);
			}
			if (ace.hasPermission(Permission.WRITE_PERMISSION)) {
				Element permElement = new Element(Hierarchy.PERMISSION);
				permElement.setAttribute(Hierarchy.PERM_NAME, Hierarchy.WRITE_PERM);
				aceElement.addContent(permElement);
			}
			if (ace.hasPermission(Permission.DELETE_PERMISSION)) {
				Element permElement = new Element(Hierarchy.PERMISSION);
				permElement.setAttribute(Hierarchy.PERM_NAME, Hierarchy.DELETE_PERM);
				aceElement.addContent(permElement);
			}
			if (ace.hasPermission(Permission.MANAGE_PERMISSION)) {
				Element permElement = new Element(Hierarchy.PERMISSION);
				permElement.setAttribute(Hierarchy.PERM_NAME, Hierarchy.MANAGE_PERM);
				aceElement.addContent(permElement);
			}
			if (ace.hasPermission(Permission.INHERIT_READ_PERMISSION)) {
				Element permElement = new Element(Hierarchy.PERMISSION);
				permElement.setAttribute(Hierarchy.PERM_NAME, Hierarchy.INHERIT_READ_PERM);
				aceElement.addContent(permElement);
			}
			if (ace.hasPermission(Permission.INHERIT_WRITE_PERMISSION)) {
				Element permElement = new Element(Hierarchy.PERMISSION);
				permElement.setAttribute(Hierarchy.PERM_NAME, Hierarchy.INHERIT_WRITE_PERM);
				aceElement.addContent(permElement);
			}
			if (ace.hasPermission(Permission.INHERIT_DELETE_PERMISSION)) {
				Element permElement = new Element(Hierarchy.PERMISSION);
				permElement.setAttribute(Hierarchy.PERM_NAME, Hierarchy.INHERIT_DELETE_PERM);
				aceElement.addContent(permElement);
			}
			if (ace.hasPermission(Permission.INHERIT_MANAGE_PERMISSION)) {
				Element permElement = new Element(Hierarchy.PERMISSION);
				permElement.setAttribute(Hierarchy.PERM_NAME, Hierarchy.INHERIT_MANAGE_PERM);
				aceElement.addContent(permElement);
			}
			aclElement.addContent(aceElement);
		}
		ret.addContent(aclElement);
		for (Iterator iter = rs.getChildren(rc).iterator(); iter.hasNext();) {
			RemoteNode rn = (RemoteNode) iter.next();
			if (rn instanceof RemoteContainer) {
				if (rn.getType().equals("trashcan")) continue;
				ret.addContent(getContainerElement((RemoteContainer)rn));
			}
		}
		return ret;
	}
}
