/*************************************************************************
 * LICENSE AGREEMENT FOR SAMPLE CODE OR ADDITIONAL FUNCTIONALITY
 * SAS INSTITUTE INC. IS PROVIDING YOU WITH THE COMPUTER SOFTWARE CODE INCLUDED WITH
 * THIS AGREEMENT ("CODE") ON AN "AS IS" BASIS, AND AUTHORIZES YOU TO USE THE CODE SUBJECT
 * TO THE TERMS HEREOF. BY USING THE CODE, YOU AGREE TO THESE TERMS. YOUR USE OF THE CODE
 * IS AT YOUR OWN RISK. SAS INSTITUTE INC. MAKES NO REPRESENTATION OR WARRANTY, EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, NONINFRINGEMENT AND TITLE, WITH RESPECT TO THE CODE.
 *
 * The Code is intended to be used solely as an example for use with part of a product ("Software")
 * you currently have licensed from SAS Institute Inc. or one of its subsidiaries or authorized
 * agents ("SAS"). The Code is designed to add functionality to the Software, but has not necessarily
 * been tested. Accordingly, SAS makes no representation or warranty that the Code will operate
 * error-free. SAS is under no obligation to maintain or support the Code.
 *
 * Neither SAS nor its licensors shall be liable to you or any third party for any general, special,
 * direct, indirect, consequential, incidental or other damages whatsoever arising out of or related
 * to your use or inability to use the Code, even if SAS has been advised of the possibility of such
 * damages.
 *
 * Except as otherwise provided above, the Code is governed by the same agreement that governs the
 * Software. If you do not have an existing agreement with SAS governing the Software, you may not
 * use the Code.
 *
 * SAS and all other SAS Institute Inc. product or service names are registered trademarks or
 * trademarks of SAS Institute Inc. in the USA and other countries. (r) indicates USA registration.
 * Other brand and product names are registered trademarks or trademarks of their respective companies.
 *
 * Copyright 2007, SAS Institute Inc., Cary, NC
 *************************************************************************/

package com.sas.drugdev.remote;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.net.URL;

import org.jdom.Element;

import com.sas.drugdev.remote.session.Session;
import com.sas.drugdev.remote.session.SessionFactory;
import com.sas.drugdev.remote.session.UsernamePasswordCredentials;

public class Hierarchy {
	public static final String PRINCIPAL = "principal";
	public static final String ACE = "ace";
	public static final String INHERIT_MANAGE_PERM = "inherit_manage";
	public static final String INHERIT_DELETE_PERM = "inherit_delete";
	public static final String INHERIT_WRITE_PERM = "inherit_write";
	public static final String INHERIT_READ_PERM = "inherit_read";
	public static final String MANAGE_PERM = "manage";
	public static final String DELETE_PERM = "delete";
	public static final String WRITE_PERM = "write";
	public static final String READ_PERM = "read";
	public static final String PERMISSION = "permission";
	public static final String PERM_NAME = "name";
	public static final String OWNER_USER = "Owner";
	public static final String ALL_USERS_GROUP = "All Users";
	public static final String ACE_USER = "user";
	public static final String ACE_GROUP = "group";
	public static final String ACE_TYPE = "type";
	public static final String BASE = "base";
	public static final String HIERARCHY = "hierarchy";
	public static final String CONTAINER = "container";
	public static final String ACL = "acl";
	public static final String PROPERTY = "property";
	public static final String OWNER = "owner";
	public static final String TYPE = "type";
	public static final String NAME = "name";
	public static final String PROPERTY_NAME = "name";
	public static final String PROPERTY_VALUE = "value";
	public static final String DEFAULT_VERSIONING = "defaultVersioning";


	private static void usage() {
		System.err.println("Usage: java com.sas.drugdev.remote.Hierarchy -create <username> <password> <url> <sdd path> <path to xml>");
		System.err.println("or java com.sas.drugdev.remote.Hierarchy -extract <username> <password> <url> <sdd path> <path to xml>");
		System.err.println("or java com.sas.drugdev.remote.Hierarchy -copy <username> <password> <url> <sdd source path> <sdd dest path>");
		System.err.println("\tIf action is \"create\" the <path to xml> is an input file, else it is an output file.");
		System.exit(-1);
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		if (args.length != 6) usage();
		String actionSwitch = args[0];
		boolean create = false, extract = false, copy = false;
		
		if ("-create".equals(actionSwitch)) {
			create = true;
		} else if ("-extract".equals(actionSwitch))  {
			extract = true;
		} else if ("-copy".equals(actionSwitch))  {
			copy = true;
		} else {
			usage();
		}
		
		String username = args[1];
		String password = args[2];
		String url = args[3];
		String sddpath = args[4];

		Session session = null;
		try {
			session = SessionFactory.newSession(new URL(url), new UsernamePasswordCredentials(username, password.toCharArray()));
			if (create) {
				String file = args[5];
				CreateTree ct = new CreateTree(session);
				ct.create(sddpath, new FileInputStream(file));
			} else if (extract) {
				String file = args[5];
				ExtractTree et = new ExtractTree(session);
				et.extract(sddpath, new FileOutputStream(file));
			} else {
				ExtractTree et = new ExtractTree(session);
				Element root = et.extract(sddpath);
				CreateTree ct = new CreateTree(session);
				String sdddestpath = args[5];
				ct.create(sdddestpath, root);
			}
		} finally {
			if (session != null) session.invalidate();
		}
	}

}
